"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AmqpTransporter = void 0;
const amqplib_1 = require("amqplib");
class AmqpTransporter {
    constructor(push_channel, listen_connection, listen_default_channel) {
        this.push_channel = push_channel;
        this.listen_connection = listen_connection;
        this.listen_default_channel = listen_default_channel;
    }
    static init(url = process.env.AMQP_TRANSPORTER) {
        return __awaiter(this, void 0, void 0, function* () {
            const push_connection = yield amqplib_1.connect(url, { reconnectTimeInSeconds: 2, heartbeatIntervalInSeconds: 1 });
            const listen_connection = yield amqplib_1.connect(url, { reconnectTimeInSeconds: 2, heartbeatIntervalInSeconds: 1 });
            const push_channel = yield push_connection.createChannel();
            const listen_default_channel = yield push_connection.createChannel();
            return new this(push_channel, listen_connection, listen_default_channel);
        });
    }
    publish(topic, data, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.push_channel.assertExchange(topic, 'topic', { autoDelete: true });
                yield this.push_channel.publish(topic, options.route, data, { replyTo: options.reply_to, messageId: options.id });
            }
            catch (e) {
                console.error(e);
            }
        });
    }
    getChannel(options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (options === null || options === void 0 ? void 0 : options.limit) {
                const channel = yield this.listen_connection.createChannel();
                channel.prefetch(options.limit, false);
                return channel;
            }
            return this.listen_default_channel;
        });
    }
    listen(topic, cb, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const channel = yield this.getChannel(options);
            const { queue } = yield channel.assertQueue(options.fanout ? '' : `${topic}${options.route || ''}`, {
                autoDelete: true
            });
            yield this.push_channel.assertExchange(topic, 'topic', { autoDelete: true });
            yield channel.bindQueue(queue, topic, options.route || '#');
            yield channel.consume(queue, (msg) => __awaiter(this, void 0, void 0, function* () {
                var _a;
                const { content, properties: { timestamp, messageId, replyTo } } = msg;
                const data = {
                    content,
                    created_time: timestamp,
                    id: messageId,
                    reply_to: replyTo,
                    delivery_attempt: ((_a = msg.properties.headers["x-death"]) === null || _a === void 0 ? void 0 : _a.length) || 0
                };
                yield cb(data);
                channel.ack(msg);
            }), { noAck: false });
            return queue;
        });
    }
}
exports.AmqpTransporter = AmqpTransporter;
